#/**********************************************
#* Execute:
#*>perl RelativeRMS.pl <ligand>.pdb <X>_transform.txt repres_<X>_transform.txt Clusters_<X>_transform.txt
#* Example:
#*>perl RelativeRMS.pl IAA_L.pdb IAA_complex_transform.txt repres_IAA_complex_transform.txt Clusters_IAA_complex_transform.txt
#***********************************************/
$pi = 3.14159265358979;
open (SOL, ">RelativeRMS.txt");
open (PDBlig, $ARGV[0])|| die "Unable to open: $!";
open (ALL,    $ARGV[1])|| die "Unable to open: $!";  # list of all solutions 
open (REPRES, $ARGV[2])|| die "Unable to open: $!"; # list of cluster representatives
open (CLUS,   $ARGV[3])|| die "Unable to open: $!"; # number of cluster to which this solution belongs
@points = <PDBlig>; # Ligant PDB
close PDBLig;
@all  = <ALL>;
close ALL;
@repres = <REPRES>;
close REPRES;
@ncl = <CLUS>; # Reading the number of the cluster od each solution
close CLUS;


# Reading the best solution
$current = $all[12];
@hex = split (" ", $current);
$NRep[0][0] = $hex[4];
$NRep[0][1] = $hex[5];
$NRep[0][2] = $hex[6];
$NRep[0][3] = $hex[7];
$NRep[0][4] = $hex[8];
$NRep[0][5] = $hex[9];
$n=1;
# Reading the solutions file
for($m=12;$m<=$#repres;$m++){       
  $current = $repres[$m];
  chomp ($current);
  @hex = split (" ", $current);
  $NRep[$n][0] = $hex[4];
  $NRep[$n][1] = $hex[5];
  $NRep[$n][2] = $hex[6];
  $NRep[$n][3] = $hex[7];
  $NRep[$n][4] = $hex[8];
  $NRep[$n++][5] = $hex[9];
}
$nAtm=0;
# Reading PDB
for ($k= 0; $k<$#points;$k++){
     $represent = $points[$k];
      chomp ($represent);
      $atom = substr$represent,0,6;
      if ($atom eq "ATOM  "){      
	  @xyz = split (" ", substr$represent,30,24);
	  $x[$nAtm]=$xyz[0];
	  $y[$nAtm]=$xyz[1];
	  $z[$nAtm]=$xyz[2];
	  $head[$nAtm] = substr$represent,0,30;
	  $tail[$nAtm++] = substr$represent,54,26;
      }
}

# Reading whole traslactions and rotations hex file
for($m=12;$m<=$#all;$m++){       
  $current = $all[$m];
  chomp ($current);
  @hex = split (" ", $current);
  # Current solution
  $tx = $hex[4];
  $ty = $hex[5];
  $tz = $hex[6];
  $a = $hex[7];
  $b = $hex[8];
  $c = $hex[9];
  $A = (cos($a) * cos($b) * cos($c)) - (sin($a) * sin($c));
  $B = -(cos($a) * cos($b) * sin($c)) - (sin($a) * cos($c));
  $C = cos($a) * sin($b);
  $D = (sin($a) * cos($b) * cos($c)) + (cos($a) * sin($c));
  $E = -(sin($a) * cos($b) * sin($c)) + (cos($a) * cos($c));
  $F = sin($a) * sin($b);
  $G = -(sin($b) * cos($c));
  $H = sin($b) * sin($c);
  $I = cos($b);
  # Representative
  $nc = $ncl[$m-11];
  $a  = $NRep[$nc][3];
  $b  = $NRep[$nc][4];
  $c  = $NRep[$nc][5];
  $Ar = (cos($a) * cos($b) * cos($c)) - (sin($a) * sin($c));
  $Br =-(cos($a) * cos($b) * sin($c)) - (sin($a) * cos($c));
  $Cr =  cos($a) * sin($b);
  $Dr = (sin($a) * cos($b) * cos($c)) + (cos($a) * sin($c));
  $Er =-(sin($a) * cos($b) * sin($c)) + (cos($a) * cos($c));
  $Fr =  sin($a) * sin($b);
  $Gr =-(sin($b) * cos($c));
  $Hr =  sin($b) * sin($c);
  $Ir =  cos($b);
  $sum=  0;
  for ($k= 0; $k<$nAtm;$k++){
      $x1 = (($x[$k] * $A) + ($y[$k] * $B) + ($z[$k] * $C))+ $tx;
      $y1 = (($x[$k] * $D) + ($y[$k] * $E) + ($z[$k] * $F))+ $ty;
      $z1 = (($x[$k] * $G) + ($y[$k] * $H) + ($z[$k] * $I))+ $tz;
      $x2 = (($x[$k] * $Ar)+ ($y[$k] * $Br)+ ($z[$k] * $Cr))+$NRep[$nc][0];
      $y2 = (($x[$k] * $Dr)+ ($y[$k] * $Er)+ ($z[$k] * $Fr))+$NRep[$nc][1];
      $z2 = (($x[$k] * $Gr)+ ($y[$k] * $Hr)+ ($z[$k] * $Ir))+$NRep[$nc][2];
      #RMS
      $x3 = ($x2-$x1)**2;
      $y3 = ($y2-$y1)**2;
      $z3 = ($z2-$z1)**2;
      $sum += $x3 + $y3 + $z3;
  }
  #RMS
  $RMS = sqrt($sum/$nAtm);
  $salido = sprintf("%i %i %4.2f %6.2f %6.2f %6.2f %6.2f %6.3f %6.3f %6.3f %i %6.2f\n",
		      $hex[0],$hex[1],$hex[2],$hex[3],$hex[4],$hex[5],$hex[6],$hex[7],$hex[8],$hex[9],$nc,$RMS);
  printf SOL $salido;    
}
close SOL;

##############################
# Funciones y Procedimientos #
##############################

sub round {
  my ($number, $places) = @_;
  my $sign = ($number < 0) ? '-' : '';
  my $abs = abs($number);

  if($places > 0) {
      $places *= 1;
      return $sign . substr($abs+("0." . "0" x $places . "5"), 0, $places+length(int($abs))+1);
  } else {
      my $p10 = 10**$places;
      return $sign . int($abs/$p10 + 0.5)*$p10;
  }
}

sub sround {
  my ($number, $length, $places) = @_;
  my $sign = ($number < 0) ? '-' : '';
  my $abs = abs($number);
  my $ent = length($sign.int($abs));
  for($i=0;$i<($length-$ent);$i++){
  $sign = " ".$sign;
  }
  if($places > 0) {
      $places *= 1;
      return $sign . substr($abs+("0." . "0" x $places . "5"),0, $places+length(int($abs))+1);
  } else {
      my $p10 = 10**$places;
      return $sign . int($abs/$p10 + 0.5)*$p10;
  }
}

sub bubblesort {
    my $array = $_[0];
    my $num   = $_[1];

    my $i;         # The initial index for the bubbling scan.
    my $j;         # The running index for the bubbling scan.
    my $ncomp = 0; # The number of comparisons.
    my $nswap = 0; # The number of swaps.

    for ( $i = $num-1; $i; $i-- ) { 
        for ( $j = 1; $j <= $i; $j++ ) {
            $ncomp++;
            # Swap if needed.
            if ( $$array[1][ $j - 1 ] < $$array[1][ $j ] ) {
		$n = $$array[1][ $j - 1 ];
		$$array[1][ $j - 1 ] = $$array[1][ $j ];
		$$array[1][ $j ] = $n;
		$n = $$array[0][ $j - 1 ];
		$$array[0][ $j - 1 ] = $$array[0][ $j ];
		$$array[0][ $j ] = $n;
                $nswap++;
            }
        }
    }
}


sub trim {
    my  $str = shift;
    $str =~ s/^\s+//;
    $str =~ s/\s+$//;
    return  $str;

}

